"""Functions for creating a logger."""

import logging
import sys

from .args import parse_args
from .loglevel import LogLevel


def setup_logger() -> None:
    """Setup the logger to perform the required logging."""
    logger = logging.getLogger()
    stream_handler = logging.StreamHandler(sys.stderr)
    file_handler = logging.FileHandler("sportsball.log")
    formatter = logging.Formatter(
        "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    )
    stream_handler.setFormatter(formatter)
    file_handler.setFormatter(formatter)
    logger.addHandler(stream_handler)
    logger.addHandler(file_handler)
    args = parse_args()
    match args.loglevel:
        case LogLevel.DEBUG:
            stream_handler.setLevel(logging.DEBUG)
            file_handler.setLevel(logging.DEBUG)
            logger.setLevel(logging.DEBUG)
        case LogLevel.INFO:
            stream_handler.setLevel(logging.INFO)
            file_handler.setLevel(logging.INFO)
            logger.setLevel(logging.INFO)
        case LogLevel.WARN:
            stream_handler.setLevel(logging.WARN)
            file_handler.setLevel(logging.WARN)
            logger.setLevel(logging.WARN)
        case LogLevel.ERROR:
            stream_handler.setLevel(logging.ERROR)
            file_handler.setLevel(logging.ERROR)
            logger.setLevel(logging.ERROR)
        case _:
            raise ValueError(f"Unrecognised loglevel: {args.loglevel}")



"""Tests for the afltables coach model class."""
import os
import unittest
import datetime

import requests_mock
from scrapesession.scrapesession import ScrapeSession
from sportsball.data.tennisabstract.tennisabstract_game_model import create_tennisabstract_game_model
from sportsball.data.league import League


class TestATPTennisAbstractGameModel(unittest.TestCase):

    def setUp(self):
        self.session = ScrapeSession(backend="memory")
        self.dir = os.path.dirname(__file__)

    def test_points(self):
        with requests_mock.Mocker() as m:
            url = "https://www.tennisabstract.com/charting/20250816-M-Cincinnati_Masters-SF-Terence_Atmane-Jannik_Sinner.html"
            with open(os.path.join(self.dir, "20250816-M-Cincinnati_Masters-SF-Terence_Atmane-Jannik_Sinner.html"), "rb") as f:
                m.get(url, content=f.read())
            with open(os.path.join(self.dir, "TerenceAtmane.html"), "rb") as f:
                m.get("https://www.tennisabstract.com/cgi-bin/player.cgi?p=TerenceAtmane", content=f.read())
            with open(os.path.join(self.dir, "JannikSinner.html"), "rb") as f:
                m.get("https://www.tennisabstract.com/cgi-bin/player.cgi?p=JannikSinner", content=f.read())
            m.get("https://historical-forecast-api.open-meteo.com/v1/forecast?latitude=39.348934&longitude=-84.27711&hourly=temperature_2m&hourly=relative_humidity_2m&hourly=dew_point_2m&hourly=apparent_temperature&hourly=precipitation&hourly=rain&hourly=snowfall&hourly=snow_depth&hourly=weather_code&hourly=pressure_msl&hourly=surface_pressure&hourly=cloud_cover&hourly=cloud_cover_low&hourly=cloud_cover_mid&hourly=cloud_cover_high&hourly=et0_fao_evapotranspiration&hourly=vapour_pressure_deficit&hourly=wind_speed_10m&hourly=wind_speed_100m&hourly=wind_direction_10m&hourly=wind_direction_100m&hourly=wind_gusts_10m&hourly=soil_temperature_0_to_7cm&hourly=soil_temperature_7_to_28cm&hourly=soil_temperature_28_to_100cm&hourly=soil_temperature_100_to_255cm&hourly=soil_moisture_0_to_7cm&hourly=soil_moisture_7_to_28cm&hourly=soil_moisture_28_to_100cm&hourly=soil_moisture_100_to_255cm&daily=weather_code&daily=temperature_2m_max&daily=temperature_2m_min&daily=temperature_2m_mean&daily=apparent_temperature_max&daily=apparent_temperature_min&daily=apparent_temperature_mean&daily=sunrise&daily=sunset&daily=daylight_duration&daily=sunshine_duration&daily=precipitation_sum&daily=rain_sum&daily=snowfall_sum&daily=precipitation_hours&daily=wind_speed_10m_max&daily=wind_gusts_10m_max&daily=wind_direction_10m_dominant&daily=shortwave_radiation_sum&daily=et0_fao_evapotranspiration&timezone=America%2FNew_York&start_date=2025-08-15&end_date=2025-08-16&format=flatbuffers")
            m.get("https://historical-forecast-api.open-meteo.com/v1/forecast?latitude=47.0&longitude=2.0&hourly=temperature_2m&hourly=relative_humidity_2m&hourly=dew_point_2m&hourly=apparent_temperature&hourly=precipitation&hourly=rain&hourly=snowfall&hourly=snow_depth&hourly=weather_code&hourly=pressure_msl&hourly=surface_pressure&hourly=cloud_cover&hourly=cloud_cover_low&hourly=cloud_cover_mid&hourly=cloud_cover_high&hourly=et0_fao_evapotranspiration&hourly=vapour_pressure_deficit&hourly=wind_speed_10m&hourly=wind_speed_100m&hourly=wind_direction_10m&hourly=wind_direction_100m&hourly=wind_gusts_10m&hourly=soil_temperature_0_to_7cm&hourly=soil_temperature_7_to_28cm&hourly=soil_temperature_28_to_100cm&hourly=soil_temperature_100_to_255cm&hourly=soil_moisture_0_to_7cm&hourly=soil_moisture_7_to_28cm&hourly=soil_moisture_28_to_100cm&hourly=soil_moisture_100_to_255cm&daily=weather_code&daily=temperature_2m_max&daily=temperature_2m_min&daily=temperature_2m_mean&daily=apparent_temperature_max&daily=apparent_temperature_min&daily=apparent_temperature_mean&daily=sunrise&daily=sunset&daily=daylight_duration&daily=sunshine_duration&daily=precipitation_sum&daily=rain_sum&daily=snowfall_sum&daily=precipitation_hours&daily=wind_speed_10m_max&daily=wind_gusts_10m_max&daily=wind_direction_10m_dominant&daily=shortwave_radiation_sum&daily=et0_fao_evapotranspiration&timezone=Europe%2FParis&start_date=2025-08-15&end_date=2025-08-16&format=flatbuffers")
            m.get("https://historical-forecast-api.open-meteo.com/v1/forecast?latitude=43.0&longitude=12.0&hourly=temperature_2m&hourly=relative_humidity_2m&hourly=dew_point_2m&hourly=apparent_temperature&hourly=precipitation&hourly=rain&hourly=snowfall&hourly=snow_depth&hourly=weather_code&hourly=pressure_msl&hourly=surface_pressure&hourly=cloud_cover&hourly=cloud_cover_low&hourly=cloud_cover_mid&hourly=cloud_cover_high&hourly=et0_fao_evapotranspiration&hourly=vapour_pressure_deficit&hourly=wind_speed_10m&hourly=wind_speed_100m&hourly=wind_direction_10m&hourly=wind_direction_100m&hourly=wind_gusts_10m&hourly=soil_temperature_0_to_7cm&hourly=soil_temperature_7_to_28cm&hourly=soil_temperature_28_to_100cm&hourly=soil_temperature_100_to_255cm&hourly=soil_moisture_0_to_7cm&hourly=soil_moisture_7_to_28cm&hourly=soil_moisture_28_to_100cm&hourly=soil_moisture_100_to_255cm&daily=weather_code&daily=temperature_2m_max&daily=temperature_2m_min&daily=temperature_2m_mean&daily=apparent_temperature_max&daily=apparent_temperature_min&daily=apparent_temperature_mean&daily=sunrise&daily=sunset&daily=daylight_duration&daily=sunshine_duration&daily=precipitation_sum&daily=rain_sum&daily=snowfall_sum&daily=precipitation_hours&daily=wind_speed_10m_max&daily=wind_gusts_10m_max&daily=wind_direction_10m_dominant&daily=shortwave_radiation_sum&daily=et0_fao_evapotranspiration&timezone=Europe%2FRome&start_date=2025-08-15&end_date=2025-08-16&format=flatbuffers")

            game_model = create_tennisabstract_game_model(
                session=self.session,
                url=url,
                league=League.ATP,
            )
            self.assertEqual(game_model.teams[0].points, 2.0)

    def test_serves_won(self):
        with requests_mock.Mocker() as m:
            url = "https://www.tennisabstract.com/charting/20250818-M-Cincinnati_Masters-F-Jannik_Sinner-Carlos_Alcaraz.html"
            with open(os.path.join(self.dir, "20250818-M-Cincinnati_Masters-F-Jannik_Sinner-Carlos_Alcaraz.html"), "rb") as f:
                m.get(url, content=f.read())
            with open(os.path.join(self.dir, "JannikSinner.html"), "rb") as f:
                m.get("https://www.tennisabstract.com/cgi-bin/player.cgi?p=JannikSinner", content=f.read())
            with open(os.path.join(self.dir, "CarlosAlcaraz.html"), "rb") as f:
                m.get("https://www.tennisabstract.com/cgi-bin/player.cgi?p=CarlosAlcaraz", content=f.read())
            m.get("https://historical-forecast-api.open-meteo.com/v1/forecast?latitude=39.348934&longitude=-84.27711&hourly=temperature_2m&hourly=relative_humidity_2m&hourly=dew_point_2m&hourly=apparent_temperature&hourly=precipitation&hourly=rain&hourly=snowfall&hourly=snow_depth&hourly=weather_code&hourly=pressure_msl&hourly=surface_pressure&hourly=cloud_cover&hourly=cloud_cover_low&hourly=cloud_cover_mid&hourly=cloud_cover_high&hourly=et0_fao_evapotranspiration&hourly=vapour_pressure_deficit&hourly=wind_speed_10m&hourly=wind_speed_100m&hourly=wind_direction_10m&hourly=wind_direction_100m&hourly=wind_gusts_10m&hourly=soil_temperature_0_to_7cm&hourly=soil_temperature_7_to_28cm&hourly=soil_temperature_28_to_100cm&hourly=soil_temperature_100_to_255cm&hourly=soil_moisture_0_to_7cm&hourly=soil_moisture_7_to_28cm&hourly=soil_moisture_28_to_100cm&hourly=soil_moisture_100_to_255cm&daily=weather_code&daily=temperature_2m_max&daily=temperature_2m_min&daily=temperature_2m_mean&daily=apparent_temperature_max&daily=apparent_temperature_min&daily=apparent_temperature_mean&daily=sunrise&daily=sunset&daily=daylight_duration&daily=sunshine_duration&daily=precipitation_sum&daily=rain_sum&daily=snowfall_sum&daily=precipitation_hours&daily=wind_speed_10m_max&daily=wind_gusts_10m_max&daily=wind_direction_10m_dominant&daily=shortwave_radiation_sum&daily=et0_fao_evapotranspiration&timezone=America%2FNew_York&start_date=2025-08-17&end_date=2025-08-18&format=flatbuffers")
            m.get("https://historical-forecast-api.open-meteo.com/v1/forecast?latitude=43.0&longitude=12.0&hourly=temperature_2m&hourly=relative_humidity_2m&hourly=dew_point_2m&hourly=apparent_temperature&hourly=precipitation&hourly=rain&hourly=snowfall&hourly=snow_depth&hourly=weather_code&hourly=pressure_msl&hourly=surface_pressure&hourly=cloud_cover&hourly=cloud_cover_low&hourly=cloud_cover_mid&hourly=cloud_cover_high&hourly=et0_fao_evapotranspiration&hourly=vapour_pressure_deficit&hourly=wind_speed_10m&hourly=wind_speed_100m&hourly=wind_direction_10m&hourly=wind_direction_100m&hourly=wind_gusts_10m&hourly=soil_temperature_0_to_7cm&hourly=soil_temperature_7_to_28cm&hourly=soil_temperature_28_to_100cm&hourly=soil_temperature_100_to_255cm&hourly=soil_moisture_0_to_7cm&hourly=soil_moisture_7_to_28cm&hourly=soil_moisture_28_to_100cm&hourly=soil_moisture_100_to_255cm&daily=weather_code&daily=temperature_2m_max&daily=temperature_2m_min&daily=temperature_2m_mean&daily=apparent_temperature_max&daily=apparent_temperature_min&daily=apparent_temperature_mean&daily=sunrise&daily=sunset&daily=daylight_duration&daily=sunshine_duration&daily=precipitation_sum&daily=rain_sum&daily=snowfall_sum&daily=precipitation_hours&daily=wind_speed_10m_max&daily=wind_gusts_10m_max&daily=wind_direction_10m_dominant&daily=shortwave_radiation_sum&daily=et0_fao_evapotranspiration&timezone=Europe%2FRome&start_date=2025-08-17&end_date=2025-08-18&format=flatbuffers")
            m.get("https://historical-forecast-api.open-meteo.com/v1/forecast?latitude=40.0&longitude=-4.0&hourly=temperature_2m&hourly=relative_humidity_2m&hourly=dew_point_2m&hourly=apparent_temperature&hourly=precipitation&hourly=rain&hourly=snowfall&hourly=snow_depth&hourly=weather_code&hourly=pressure_msl&hourly=surface_pressure&hourly=cloud_cover&hourly=cloud_cover_low&hourly=cloud_cover_mid&hourly=cloud_cover_high&hourly=et0_fao_evapotranspiration&hourly=vapour_pressure_deficit&hourly=wind_speed_10m&hourly=wind_speed_100m&hourly=wind_direction_10m&hourly=wind_direction_100m&hourly=wind_gusts_10m&hourly=soil_temperature_0_to_7cm&hourly=soil_temperature_7_to_28cm&hourly=soil_temperature_28_to_100cm&hourly=soil_temperature_100_to_255cm&hourly=soil_moisture_0_to_7cm&hourly=soil_moisture_7_to_28cm&hourly=soil_moisture_28_to_100cm&hourly=soil_moisture_100_to_255cm&daily=weather_code&daily=temperature_2m_max&daily=temperature_2m_min&daily=temperature_2m_mean&daily=apparent_temperature_max&daily=apparent_temperature_min&daily=apparent_temperature_mean&daily=sunrise&daily=sunset&daily=daylight_duration&daily=sunshine_duration&daily=precipitation_sum&daily=rain_sum&daily=snowfall_sum&daily=precipitation_hours&daily=wind_speed_10m_max&daily=wind_gusts_10m_max&daily=wind_direction_10m_dominant&daily=shortwave_radiation_sum&daily=et0_fao_evapotranspiration&timezone=Europe%2FMadrid&start_date=2025-08-17&end_date=2025-08-18&format=flatbuffers")

            game_model = create_tennisabstract_game_model(
                session=self.session,
                url=url,
                league=League.ATP,
            )
            self.assertEqual(game_model.teams[0].players[0].serves_won, 100.0)



"""NBA NBA.com league model."""

# pylint: disable=too-many-statements,protected-access,too-many-locals,bare-except,too-many-branches,duplicate-code
import datetime
import logging
from typing import Iterator

from scrapesession.scrapesession import ScrapeSession  # type: ignore

from ...game_model import VERSION, GameModel
from ...league import League
from ...league_model import SHUTDOWN_FLAG, LeagueModel, needs_shutdown
from .nba_nbacom_game_model import create_nba_nbacom_game_model


class NBANBAComLeagueModel(LeagueModel):
    """NBA NBA.com implementation of the league model."""

    def __init__(self, session: ScrapeSession, position: int | None = None) -> None:
        super().__init__(League.AFL, session, position=position)

    @classmethod
    def name(cls) -> str:
        return "nba-nbacom-league-model"

    @property
    def games(self) -> Iterator[GameModel]:
        try:
            with self.session.cache_disabled():
                date_slug = datetime.datetime.today().date().strftime("%Y%m%d")
                response = self.session.get(
                    f"https://stats.nba.com/js/data/leaders/00_daily_lineups_{date_slug}.json"
                )
                if response.ok:
                    lineup = response.json()
                    for game in lineup["games"]:
                        if needs_shutdown():
                            return
                        yield create_nba_nbacom_game_model(
                            game=game, session=self.session, version=VERSION
                        )
                else:
                    logging.warning("Failed to fetch NBA daily lineups.")
        except Exception as exc:
            SHUTDOWN_FLAG.set()
            raise exc



"""Tests for the sportsdb game model class."""
import datetime
import os
import unittest

import requests_mock
from scrapesession.scrapesession import ScrapeSession
from sportsball.data.sportsdb.sportsdb_game_model import create_sportsdb_game_model
from sportsball.data.league import League
from sportsball.data.season_type import SeasonType


class TestSportsDBVenueModel(unittest.TestCase):

    def setUp(self):
        self._session = ScrapeSession(backend="memory")
        self.dir = os.path.dirname(__file__)

    def test_identifier(self):
        with requests_mock.Mocker() as m:
            with open(os.path.join(self.dir, "venue_16094.json"), "rb") as f:
                m.get("https://www.thesportsdb.com/api/v1/json/3/lookupvenue.php?id=16094", content=f.read())
            m.get("https://historical-forecast-api.open-meteo.com/v1/forecast?latitude=39.9013695&longitude=-75.1700964&start_date=2023-09-14&end_date=2023-09-15&hourly=temperature_2m&hourly=relative_humidity_2m&hourly=dew_point_2m&hourly=apparent_temperature&hourly=precipitation&hourly=rain&hourly=snowfall&hourly=snow_depth&hourly=weather_code&hourly=pressure_msl&hourly=surface_pressure&hourly=cloud_cover&hourly=cloud_cover_low&hourly=cloud_cover_mid&hourly=cloud_cover_high&hourly=et0_fao_evapotranspiration&hourly=vapour_pressure_deficit&hourly=wind_speed_10m&hourly=wind_speed_100m&hourly=wind_direction_10m&hourly=wind_direction_100m&hourly=wind_gusts_10m&hourly=soil_temperature_0_to_7cm&hourly=soil_temperature_7_to_28cm&hourly=soil_temperature_28_to_100cm&hourly=soil_temperature_100_to_255cm&hourly=soil_moisture_0_to_7cm&hourly=soil_moisture_7_to_28cm&hourly=soil_moisture_28_to_100cm&hourly=soil_moisture_100_to_255cm&daily=weather_code&daily=temperature_2m_max&daily=temperature_2m_min&daily=temperature_2m_mean&daily=apparent_temperature_max&daily=apparent_temperature_min&daily=apparent_temperature_mean&daily=sunrise&daily=sunset&daily=daylight_duration&daily=sunshine_duration&daily=precipitation_sum&daily=rain_sum&daily=snowfall_sum&daily=precipitation_hours&daily=wind_speed_10m_max&daily=wind_gusts_10m_max&daily=wind_direction_10m_dominant&daily=shortwave_radiation_sum&daily=et0_fao_evapotranspiration&timezone=America%2FNew_York&format=flatbuffers")
            m.get("https://news.google.com/rss/search?q=%22Philadelphia+Eagles%22+%2B+%28sport+OR+nfl+OR+%22National+Football+League%22%29+after%3A2023-09-13+before%3A2023-09-14&ceid=US:en&hl=en&gl=US")
            m.get("https://news.google.com/rss/search?q=%22Minnesota+Vikings%22+%2B+%28sport+OR+nfl+OR+%22National+Football+League%22%29+after%3A2023-09-13+before%3A2023-09-14&ceid=US:en&hl=en&gl=US")
            game_model = create_sportsdb_game_model(
                self._session,
                {
                    "strTimestamp": "2023-09-15T00:15:00",
                    "idVenue": "16094",
                    "intHomeScore": "34",
                    "intAwayScore": "28",
                    "idHomeTeam": "134936",
                    "strHomeTeam": "Philadelphia Eagles",
                    "idAwayTeam": "134941",
                    "strAwayTeam": "Minnesota Vikings",
                },
                0,
                0,
                League.NFL,
                2019,
                SeasonType.REGULAR,
            )
            self.assertEqual(game_model.dt, datetime.datetime(2023, 9, 15, 0, 15))



"""ESPN umpire model."""

# pylint: disable=duplicate-code
import datetime

import pytest_is_running
import requests_cache

from ...cache import MEMORY
from ..umpire_model import VERSION, UmpireModel


def _create_espn_umpire_model(
    session: requests_cache.CachedSession,
    url: str,
    version: str,
) -> UmpireModel:
    response = session.get(url)
    response.raise_for_status()
    data = response.json()
    name = (
        " ".join([data["firstName"], data["lastName"]])
        if "firstName" in data
        else data["displayName"]
    )
    return UmpireModel(
        identifier=data.get("id", data.get("displayName", name)),
        name=name,
        birth_date=None,
        age=None,
        birth_address=None,
        high_school=None,
        version=version,
    )


@MEMORY.cache(ignore=["session"])
def _cached_create_espn_umpire_model(
    session: requests_cache.CachedSession,
    url: str,
    version: str,
) -> UmpireModel:
    return _create_espn_umpire_model(session=session, url=url, version=version)


def create_espn_umpire_model(
    session: requests_cache.CachedSession,
    dt: datetime.datetime,
    url: str,
) -> UmpireModel:
    """Create umpire model from ESPN."""
    if (
        not pytest_is_running.is_running()
        and dt.date() < datetime.datetime.today().date() - datetime.timedelta(days=7)
    ):
        return _cached_create_espn_umpire_model(
            session=session, url=url, version=VERSION
        )
    with session.cache_disabled():
        return _create_espn_umpire_model(session=session, url=url, version=VERSION)



"""Function for normalising names."""

import re
import unicodedata

REGEX = re.compile("[^a-zA-Z]")


def normalise_name(name: str) -> str:
    """Handles Surname, Firstname"""
    if "," in name:
        name = " ".join(reversed([x.strip() for x in name.split(",")]))
    return REGEX.sub("", unicodedata.normalize("NFC", name).lower().strip())



"""The main sportsball class for accessing data."""

# pylint: disable=too-many-branches
from typing import Dict
from warnings import simplefilter

import pandas as pd
from dotenv import load_dotenv
from scrapesession.scrapesession import ScrapeSession  # type: ignore
from scrapesession.scrapesession import create_scrape_session

from .data.afl import AFLLeagueModel
from .data.aflw import AFLWLeagueModel
from .data.atp import ATPLeagueModel
from .data.epl import EPLLeagueModel
from .data.fifa import FIFALeagueModel
from .data.hkjc import HKJCLeagueModel
from .data.ipl import IPLLeagueModel
from .data.league import League
from .data.league_model import LeagueModel
from .data.mlb import MLBLeagueModel
from .data.nba import NBALeagueModel
from .data.ncaab import NCAABLeagueModel
from .data.ncaaf import NCAAFLeagueModel
from .data.nfl import NFLLeagueModel
from .data.nhl import NHLLeagueModel
from .data.wnba import WNBALeagueModel
from .data.wta import WTALeagueModel


class SportsBall:
    """The main sportsball class."""

    # pylint: disable=too-few-public-methods

    _leagues: Dict[str, LeagueModel]
    _session: ScrapeSession

    def __init__(self) -> None:
        self._session = create_scrape_session(
            "sportsball",
            {
                "https://news.google.com/",
                "https://historical-forecast-api.open-meteo.com/",
                "https://api.open-meteo.com/",
            },
        )
        self._leagues = {}
        simplefilter(action="ignore", category=pd.errors.PerformanceWarning)
        load_dotenv()

    def league(self, league: League, league_filter: str | None) -> LeagueModel:
        """Provide a league model for the given league."""
        if league not in self._leagues:
            if league == League.NFL:
                self._leagues[league] = NFLLeagueModel(self._session, league_filter)
            elif league == League.AFL:
                self._leagues[league] = AFLLeagueModel(self._session, league_filter)
            elif league == League.NBA:
                self._leagues[league] = NBALeagueModel(self._session, league_filter)
            elif league == League.NCAAF:
                self._leagues[league] = NCAAFLeagueModel(self._session, league_filter)
            elif league == League.NCAAB:
                self._leagues[league] = NCAABLeagueModel(self._session, league_filter)
            elif league == League.HKJC:
                self._leagues[league] = HKJCLeagueModel(self._session)
            elif league == League.NHL:
                self._leagues[league] = NHLLeagueModel(self._session, league_filter)
            elif league == League.MLB:
                self._leagues[league] = MLBLeagueModel(self._session, league_filter)
            elif league == League.EPL:
                self._leagues[league] = EPLLeagueModel(self._session, league_filter)
            elif league == League.IPL:
                self._leagues[league] = IPLLeagueModel(self._session, league_filter)
            elif league == League.FIFA:
                self._leagues[league] = FIFALeagueModel(self._session, league_filter)
            elif league == League.ATP:
                self._leagues[league] = ATPLeagueModel(self._session, league_filter)
            elif league == League.WNBA:
                self._leagues[league] = WNBALeagueModel(self._session, league_filter)
            elif league == League.AFLW:
                self._leagues[league] = AFLWLeagueModel(self._session, league_filter)
            elif league == League.WTA:
                self._leagues[league] = WTALeagueModel(self._session, league_filter)
            else:
                raise ValueError(f"Unrecognised league: {league}")
        return self._leagues[league]



"""Tests for the oddsportal league model class."""
import os
import unittest

import requests_cache
from sportsball.data.oddsportal.oddsportal_league_model import _find_ids


class TestOddsPortalLeagueModel(unittest.TestCase):

    def setUp(self):
        self.session = requests_cache.CachedSession(backend="memory")
        self.dir = os.path.dirname(__file__)

    def test_current_html_ids(self):
        with open(os.path.join(self.dir, "results.html"), "r") as handle:
            sports_id, oddsportal_id = _find_ids(handle.read())
            self.assertEqual(sports_id, "18")
            self.assertEqual(oddsportal_id, "6y3HOkJ7")



"""The prototype class for news."""

import datetime
from typing import Literal

from pydantic import BaseModel, ConfigDict, Field

from .field_type import TYPE_KEY, FieldType

NEWS_SUMMARY_COLUMN: Literal["summary"] = "summary"
NEWS_TITLE_COLUMN: Literal["title"] = "title"
NEWS_PUBLISHED_COLUMN: Literal["published"] = "published"
NEWS_SOURCE_COLUMN: Literal["source"] = "source"


class NewsModel(BaseModel):
    """The serialisable news class."""

    model_config = ConfigDict(
        validate_assignment=False,
        revalidate_instances="never",
        extra="ignore",
        from_attributes=False,
    )

    title: str = Field(
        ..., json_schema_extra={TYPE_KEY: FieldType.TEXT}, alias=NEWS_TITLE_COLUMN
    )
    published: datetime.datetime = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.DATETIME},
        alias=NEWS_PUBLISHED_COLUMN,
    )
    summary: str = Field(
        ..., json_schema_extra={TYPE_KEY: FieldType.TEXT}, alias=NEWS_SUMMARY_COLUMN
    )
    source: str = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.CATEGORICAL},
        alias=NEWS_SOURCE_COLUMN,
    )



"""The enumeration of the different supported positions."""

from enum import StrEnum


class Position(StrEnum):
    """An enumeration over the different positions."""

    LINE_BACKER = "LB"
    DEFENSIVE_TACKLE = "DT"
    QUARTER_BACK = "QB"
    DEFENSIVE_LINESMAN = "DL"
    OFFENSIVE_LINESMAN = "OL"
    SAFETY = "S"
    WIDE_RECEIVER = "WR"
    GUARD = "G"
    RUNNING_BACK = "RB"
    TIGHT_END = "TE"
    OFFENSIVE_TACKLE = "OT"
    CORNER_BACK = "CB"
    DEFENSIVE_BACK = "DB"
    DEFENSIVE_END = "DE"
    EDGE_RUSHER = "EDGE"
    PUNTER = "P"
    LONG_SNAPPER = "LS"
    PLACE_KICKER = "PK"
    CENTER = "C"
    FULL_BACK = "FB"
    NOSE_TACKLE = "NT"
    ATHLETE = "ATH"
    KICK_RETURNER = "KR"
    PUNT_RETURNER = "PR"



"""ESPN coach model."""

import datetime
from typing import Any

import pytest_is_running
import requests_cache

from ...cache import MEMORY
from ..coach_model import VERSION, CoachModel


def _create_espn_coach_model(
    session: requests_cache.CachedSession,
    url: str,
    version: str,
) -> dict[str, Any]:
    response = session.get(url)
    response.raise_for_status()
    data = response.json()
    return CoachModel(
        identifier=data["id"],
        name=" ".join([data["firstName"], data["lastName"]]),
        birth_date=None,
        age=None,
        version=version,
    ).model_dump()


@MEMORY.cache(ignore=["session"])
def _cached_create_espn_coach_model(
    session: requests_cache.CachedSession,
    url: str,
    version: str,
) -> dict[str, Any]:
    return _create_espn_coach_model(session=session, url=url, version=version)


def create_espn_coach_model(
    session: requests_cache.CachedSession,
    dt: datetime.datetime,
    url: str,
) -> CoachModel:
    """Create coach model from ESPN."""
    if (
        not pytest_is_running.is_running()
        and dt.date() < datetime.datetime.today().date() - datetime.timedelta(days=7)
    ):
        return CoachModel.model_construct(
            **_cached_create_espn_coach_model(session=session, url=url, version=VERSION)
        )
    with session.cache_disabled():
        return CoachModel.model_construct(
            **_create_espn_coach_model(session=session, url=url, version=VERSION)
        )



"""The prototype class for a game."""

import datetime
from typing import Literal

import pytz
from pydantic import BaseModel, ConfigDict, Field

from .delimiter import DELIMITER
from .dividend_model import DividendModel
from .field_type import TYPE_KEY, FieldType
from .season_type import SeasonType
from .team_model import VERSION as TEAM_VERSION
from .team_model import TeamModel
from .umpire_model import VERSION as UMPIRE_VERSION
from .umpire_model import UmpireModel
from .venue_model import VERSION as VENUE_VERSION
from .venue_model import VenueModel

GAME_DT_COLUMN = "dt"
SEASON_TYPE_COLUMN = "season_type"
SEASON_YEAR_COLUMN = "year"
VENUE_COLUMN_PREFIX: Literal["venue"] = "venue"
TEAM_COLUMN_PREFIX: Literal["teams"] = "teams"
GAME_ATTENDANCE_COLUMN: Literal["attendance"] = "attendance"
GAME_WEEK_COLUMN: Literal["week"] = "week"
LEAGUE_COLUMN: Literal["league"] = "league"
END_DT_COLUMN: Literal["end_dt"] = "end_dt"
POSTPONED_COLUMN: Literal["postponed"] = "postponed"
PLAY_OFF_COLUMN: Literal["play_off"] = "play_off"
GAME_DISTANCE_COLUMN: Literal["distance"] = "distance"
GAME_DIVIDENDS_COLUMN: Literal["dividends"] = "dividends"
GAME_POT_COLUMN: Literal["pot"] = "pot"
GAME_UMPIRES_COLUMN: Literal["umpires"] = "umpires"
VERSION = DELIMITER.join(["0.0.1", VENUE_VERSION, TEAM_VERSION, UMPIRE_VERSION])


def localize(venue: VenueModel | None, dt: datetime.datetime) -> datetime.datetime:
    """Localize the datetime against the timezone of the venue."""
    if dt.tzinfo is not None and dt.tzinfo.utcoffset(dt) is not None:
        return dt
    tz = None
    venue_model = venue
    if venue_model is not None:
        address = venue_model.address
        if address is not None:
            timezone = address.timezone
            if timezone is not None:
                tz = pytz.timezone(timezone)
    if tz is None:
        tz = pytz.utc
    return tz.localize(dt)


class GameModel(BaseModel):
    """The serialisable representation of a game."""

    model_config = ConfigDict(
        validate_assignment=False,
        revalidate_instances="never",
        extra="ignore",
        from_attributes=False,
    )

    dt: datetime.datetime
    week: int | None = Field(..., alias=GAME_WEEK_COLUMN)
    game_number: int | None
    venue: VenueModel | None = Field(..., alias=VENUE_COLUMN_PREFIX)
    teams: list[TeamModel] = Field(..., alias=TEAM_COLUMN_PREFIX)
    end_dt: datetime.datetime | None = Field(
        ..., json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD}, alias=END_DT_COLUMN
    )
    attendance: int | None = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=GAME_ATTENDANCE_COLUMN,
    )
    league: str = Field(..., alias=LEAGUE_COLUMN)
    year: int | None
    season_type: SeasonType | None
    postponed: bool | None = Field(
        ..., json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD}, alias=POSTPONED_COLUMN
    )
    play_off: bool | None = Field(..., alias=PLAY_OFF_COLUMN)
    distance: float | None = Field(..., alias=GAME_DISTANCE_COLUMN)
    dividends: list[DividendModel] = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=GAME_DIVIDENDS_COLUMN,
    )
    pot: float | None = Field(
        ..., json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD}, alias=GAME_POT_COLUMN
    )
    version: str = Field(..., json_schema_extra={TYPE_KEY: FieldType.CATEGORICAL})
    umpires: list[UmpireModel] = Field(
        ...,
        alias=GAME_UMPIRES_COLUMN,
    )



"""Tests for the sportsdb league model class."""
import unittest

import requests_cache
from sportsball.data.sportsdb.sportsdb_league_model import SportsDBLeagueModel
from sportsball.data.league import League


class TestSportsDBLeagueModel(unittest.TestCase):

    def setUp(self):
        self.session = requests_cache.CachedSession(backend="memory")
        self.model = SportsDBLeagueModel(self.session, "", League.NBA)

    def test_league(self):
        self.assertEqual(self.model.league, League.NBA)



"""Setup sportsball."""
from setuptools import setup, find_packages
from pathlib import Path
import typing

readme_path = Path(__file__).absolute().parent.joinpath('README.md')
long_description = readme_path.read_text(encoding='utf-8')


def install_requires() -> typing.List[str]:
    """Find the install requires strings from requirements.txt"""
    requires = []
    with open(
        Path(__file__).absolute().parent.joinpath('requirements.txt'), "r"
    ) as requirments_txt_handle:
        for require in requirments_txt_handle:
            if not require.startswith(".") and not require.startswith("-e"):
                requires.append(require)
            else:
                require_file = require.split()[-1]
                require_file = require_file.replace("git+", "")
                package_name = require_file.split("#egg=")[-1]
                requires.append(package_name + " @ " + require_file)
    return requires


setup(
    name='sportsball',
    version='0.30.5',
    description='A library for pulling in and normalising sports stats.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
    ],
    keywords='sports data betting',
    url='https://github.com/8W9aG/sportsball',
    author='Will Sackfield',
    author_email='will.sackfield@gmail.com',
    license='MIT',
    install_requires=install_requires(),
    zip_safe=False,
    packages=find_packages(),
    entry_points = {
        'console_scripts': ['sportsball=sportsball.__main__:main'],
    },
    include_package_data=True,
)



"""TennisAbstract game model."""

# pylint: disable=duplicate-code,too-many-statements,too-many-branches,too-many-locals,too-many-nested-blocks
import io
import logging
import os
import urllib
import urllib.parse
import warnings
from urllib.parse import urlparse

import numpy as np
import pandas as pd
import pytest_is_running
from bs4 import BeautifulSoup
from dateutil.parser import parse
from scrapesession.scrapesession import ScrapeSession  # type: ignore

from ...cache import MEMORY
from ..game_model import VERSION, GameModel
from ..league import League
from .tennisabstract_team_model import create_tennisabstract_team_model
from .tennisabstract_venue_model import create_tennisabstract_venue_model


def _ratio_string(ratio: str) -> float:
    ratio_split = ratio.split("/")
    try:
        return float(ratio_split[0]) / float(ratio_split[1])
    except ZeroDivisionError:
        return 0.0


def _create_tennisabstract_game_model(
    session: ScrapeSession,
    url: str,
    league: League,
    version: str,
) -> GameModel:
    warnings.simplefilter("ignore", category=FutureWarning)
    try:
        o = urlparse(url)
        filename = os.path.basename(o.path)
        dt = parse(filename.split("-")[0])

        with session.wayback_disabled():
            response = session.get(url)
        response.raise_for_status()
        soup = BeautifulSoup(response.text, "lxml")

        venue = None
        for h2 in soup.find_all("h2"):
            game_title = " ".join(h2.get_text().strip().split(":")[0].split()[1:-1])
            venue = create_tennisabstract_venue_model(game_title, session, dt)

        points = [0 for _ in range(2)]
        for b in soup.find_all("b"):
            description = b.get_text().strip()
            for word in description.split():
                if "-" not in word:
                    continue
                word_points = np.array(
                    [int(x.split("(")[0].strip()) for x in word.split("-")]
                )
                points[np.argmax(word_points)] += 1

        ace_percentages = []
        double_fault_percentages = []
        first_serves_ins = []
        first_serve_percentages = []
        second_serve_percentages = []
        break_points_saveds = []
        return_points_won_percentages = []
        winners = []
        winners_fronthands = []
        winners_backhands = []
        unforced_errors = []
        unforced_errors_fronthand = []
        unforced_errors_backhands = []
        serve_points = []
        serves_won = []
        serves_aces = []
        serves_unreturned = []
        serves_forced_error_percentage = []
        serves_won_in_three_shots_or_less = []
        serves_wide_percentage = []
        serves_body_percentage = []
        serves_t_percentage = []
        serves_wide_deuce_percentage = []
        serves_body_deuce_percentage = []
        serves_t_deuce_percentage = []
        serves_wide_ad_percentage = []
        serves_body_ad_percentage = []
        serves_t_ad_percentage = []
        serves_net_percentage = []
        serves_wide_direction_percentage = []
        shots_deep_percentage = []
        shots_deep_wide_percentage = []
        shots_foot_errors_percentage = []
        shots_unknown_percentage = []
        points_won_percentage = []
        for script in soup.find_all("script"):
            script_text = script.get_text().strip()
            if "<table" not in script_text:
                continue
            for line in script_text.split("\n"):
                if "<table" not in line:
                    continue
                html_text = ("'<" + line.split("'<")[-1]).split("';")[0]
                handle = io.StringIO()
                handle.write(html_text)
                handle.seek(0)
                dfs = pd.read_html(handle)
                for df in dfs:
                    cols = set(df.columns.values.tolist())
                    found_total = False
                    for val in df[df.columns.values.tolist()[0]]:
                        if isinstance(val, str) and "Total" in val:
                            found_total = True
                            break
                    if found_total:
                        df = df[
                            df[df.columns.values.tolist()[0]]
                            .str.contains("Total")
                            .fillna(False)
                        ]
                    if "A%" in cols:
                        ace_percentages = [
                            float(x.replace("%", ""))
                            for x in df["A%"].dropna().tolist()
                        ]
                    if "DF%" in cols:
                        double_fault_percentages = [
                            float(x.replace("%", ""))
                            for x in df["DF%"].dropna().tolist()
                        ]
                    if "1stIn" in cols:
                        first_serves_ins = [
                            float(x.split("(")[0].strip().replace("%", ""))
                            for x in df["1stIn"].dropna().tolist()
                        ]
                    if "1st%" in cols:
                        first_serve_percentages = [
                            float(x.replace("%", ""))
                            for x in df["1st%"].dropna().tolist()
                        ]
                    if "2nd%" in cols:
                        second_serve_percentages = [
                            float(x.replace("%", ""))
                            for x in df["2nd%"].dropna().tolist()
                        ]
                    if "BPSaved" in cols:
                        break_points_saveds = [
                            _ratio_string(x) for x in df["BPSaved"].dropna().tolist()
                        ]
                    if "RPW%" in cols:
                        return_points_won_percentages = [
                            float(x.replace("%", ""))
                            for x in df["RPW%"].dropna().tolist()
                        ]
                    if "Winners (FH/BH)" in cols:
                        winners = [
                            int(x.split("(")[0].strip())
                            for x in df["Winners (FH/BH)"].dropna().tolist()
                        ]
                        winners_fronthands = [
                            int(x.split("(")[1].split("/")[0].strip())
                            for x in df["Winners (FH/BH)"].dropna().tolist()
                        ]
                        winners_backhands = [
                            int(
                                x.split("(")[1]
                                .split("/")[1]
                                .strip()
                                .split(")")[0]
                                .strip()
                            )
                            for x in df["Winners (FH/BH)"].dropna().tolist()
                        ]
                    if "UFE (FH/BH)" in cols:
                        unforced_errors = [
                            int(x.split("(")[0].strip())
                            for x in df["UFE (FH/BH)"].dropna().tolist()
                        ]
                        unforced_errors_fronthand = [
                            int(x.split("(")[1].split("/")[0].strip())
                            for x in df["UFE (FH/BH)"].dropna().tolist()
                        ]
                        unforced_errors_backhands = [
                            int(
                                x.split("(")[1]
                                .split("/")[1]
                                .strip()
                                .split(")")[0]
                                .strip()
                            )
                            for x in df["UFE (FH/BH)"].dropna().tolist()
                        ]
                    if "Pts" in cols:
                        serve_points = [int(x) for x in df["Pts"].dropna().tolist()]
                    if "Won---%" in cols:
                        serves_won = [
                            float(
                                x.split("(")[-1]
                                .strip()
                                .replace("%", "")
                                .replace(")", "")
                                .replace("-", "0")
                            )
                            for x in df["Won---%"].dropna().tolist()
                        ]
                    if "Aces---%" in cols:
                        serves_aces = [
                            float(
                                x.split("(")[-1]
                                .strip()
                                .replace("%", "")
                                .replace(")", "")
                                .replace("-", "0")
                            )
                            for x in df["Aces---%"].dropna().tolist()
                        ]
                    if "Unret---%" in cols:
                        serves_unreturned = [
                            float(
                                x.split("(")[-1]
                                .strip()
                                .replace("%", "")
                                .replace(")", "")
                            )
                            for x in df["Unret---%"].dropna().tolist()
                        ]
                    if "FcdE---%" in cols:
                        serves_forced_error_percentage = [
                            float(
                                x.split("(")[-1]
                                .strip()
                                .replace("%", "")
                                .replace(")", "")
                                .replace("-", "0")
                            )
                            for x in df["FcdE---%"].dropna().tolist()
                        ]
                    if "<=3W---%" in cols:
                        serves_won_in_three_shots_or_less = [
                            float(
                                x.split("(")[-1]
                                .strip()
                                .replace("%", "")
                                .replace(")", "")
                                .replace("-", "0")
                            )
                            for x in df["<=3W---%"].dropna().tolist()
                        ]
                    if "Wide---%" in cols:
                        serves_wide_percentage = [
                            float(
                                x.split("(")[-1]
                                .strip()
                                .replace("%", "")
                                .replace(")", "")
                            )
                            for x in df["Wide---%"].dropna().tolist()
                        ]
                    if "Body---%" in cols:
                        serves_body_percentage = [
                            float(
                                x.split("(")[-1]
                                .strip()
                                .replace("%", "")
                                .replace(")", "")
                            )
                            for x in df["Body---%"].dropna().tolist()
                        ]
                    if "T---%" in cols:
                        serves_t_percentage = [
                            float(
                                x.split("(")[-1]
                                .strip()
                                .replace("%", "")
                                .replace(")", "")
                            )
                            for x in df["T---%"].dropna().tolist()
                        ]
                    if "Dc-Wide-%" in cols:
                        serves_wide_deuce_percentage = [
                            float(
                                x.split("(")[-1]
                                .strip()
                                .replace("%", "")
                                .replace(")", "")
                            )
                            for x in df["Dc-Wide-%"].dropna().tolist()
                        ]
                    if "Dc-Body-%" in cols:
                        serves_body_deuce_percentage = [
                            float(
                                x.split("(")[-1]
                                .strip()
                                .replace("%", "")
                                .replace(")", "")
                            )
                            for x in df["Dc-Body-%"].dropna().tolist()
                        ]
                    if "Dc-T---%" in cols:
                        serves_t_deuce_percentage = [
                            float(
                                x.split("(")[-1]
                                .strip()
                                .replace("%", "")
                                .replace(")", "")
                            )
                            for x in df["Dc-T---%"].dropna().tolist()
                        ]
                    if "Ad-Wide-%" in cols:
                        serves_wide_ad_percentage = [
                            float(
                                x.split("(")[-1]
                                .strip()
                                .replace("%", "")
                                .replace(")", "")
                            )
                            for x in df["Ad-Wide-%"].dropna().tolist()
                        ]
                    if "Ad-Body-%" in cols:
                        serves_body_ad_percentage = [
                            float(
                                x.split("(")[-1]
                                .strip()
                                .replace("%", "")
                                .replace(")", "")
                            )
                            for x in df["Ad-Body-%"].dropna().tolist()
                        ]
                    if "Ad-T---%" in cols:
                        serves_t_ad_percentage = [
                            float(
                                x.split("(")[-1]
                                .strip()
                                .replace("%", "")
                                .replace(")", "")
                            )
                            for x in df["Ad-T---%"].dropna().tolist()
                        ]
                    if "net---%" in cols:
                        serves_net_percentage = [
                            float(
                                x.split("(")[-1]
                                .strip()
                                .replace("%", "")
                                .replace(")", "")
                            )
                            for x in df["net---%"].dropna().tolist()
                        ]
                    if "wide---%" in cols:
                        serves_wide_direction_percentage = [
                            float(
                                x.split("(")[-1]
                                .strip()
                                .replace("%", "")
                                .replace(")", "")
                            )
                            for x in df["wide---%"].dropna().tolist()
                        ]
                    if "deep---%" in cols:
                        shots_deep_percentage = [
                            float(
                                x.split("(")[-1]
                                .strip()
                                .replace("%", "")
                                .replace(")", "")
                            )
                            for x in df["deep---%"].dropna().tolist()
                        ]
                    if "w&d---%" in cols:
                        shots_deep_wide_percentage = [
                            float(
                                x.split("(")[-1]
                                .strip()
                                .replace("%", "")
                                .replace(")", "")
                            )
                            for x in df["w&d---%"].dropna().tolist()
                        ]
                    if "foot---%" in cols:
                        shots_foot_errors_percentage = [
                            float(
                                x.split("(")[-1]
                                .strip()
                                .replace("%", "")
                                .replace(")", "")
                            )
                            for x in df["foot---%"].dropna().tolist()
                        ]
                    if "unk---%" in cols:
                        shots_unknown_percentage = [
                            float(
                                x.split("(")[-1]
                                .strip()
                                .replace("%", "")
                                .replace(")", "")
                            )
                            for x in df["unk---%"].dropna().tolist()
                        ]
                    if "PtsW----%" in cols:
                        points_won_percentage = [
                            float(
                                x.split("(")[-1]
                                .strip()
                                .replace("%", "")
                                .replace(")", "")
                            )
                            for x in df["PtsW----%"].dropna().tolist()
                        ]

        player_urls = []
        for a in soup.find_all("a"):
            player_url = urllib.parse.urljoin(url, a.get("href"))
            o = urlparse(player_url)
            filename = os.path.basename(o.path)
            if filename.endswith("player.cgi"):
                player_urls.append(player_url)

        return GameModel(
            dt=dt,
            week=None,
            game_number=None,
            venue=venue,
            teams=[
                create_tennisabstract_team_model(
                    session=session,
                    dt=dt,
                    league=league,
                    player_urls=[x],
                    ace_percentages=ace_percentages,
                    double_fault_percentages=double_fault_percentages,
                    first_serves_ins=first_serves_ins,
                    first_serve_percentages=first_serve_percentages,
                    second_serve_percentages=second_serve_percentages,
                    break_points_saveds=break_points_saveds,
                    return_points_won_percentages=return_points_won_percentages,
                    winners=winners,
                    winners_fronthands=winners_fronthands,
                    winners_backhands=winners_backhands,
                    unforced_errors=unforced_errors,
                    unforced_errors_fronthand=unforced_errors_fronthand,
                    unforced_errors_backhand=unforced_errors_backhands,
                    serve_points=serve_points,
                    serves_won=serves_won,
                    serves_aces=serves_aces,
                    serves_unreturned=serves_unreturned,
                    serves_forced_error_percentage=serves_forced_error_percentage,
                    serves_won_in_three_shots_or_less=serves_won_in_three_shots_or_less,
                    serves_wide_percentage=serves_wide_percentage,
                    serves_body_percentage=serves_body_percentage,
                    serves_t_percentage=serves_t_percentage,
                    serves_wide_deuce_percentage=serves_wide_deuce_percentage,
                    serves_body_deuce_percentage=serves_body_deuce_percentage,
                    serves_t_deuce_percentage=serves_t_deuce_percentage,
                    serves_wide_ad_percentage=serves_wide_ad_percentage,
                    serves_body_ad_percentage=serves_body_ad_percentage,
                    serves_t_ad_percentage=serves_t_ad_percentage,
                    serves_net_percentage=serves_net_percentage,
                    serves_wide_direction_percentage=serves_wide_direction_percentage,
                    shots_deep_percentage=shots_deep_percentage,
                    shots_deep_wide_percentage=shots_deep_wide_percentage,
                    shots_foot_errors_percentage=shots_foot_errors_percentage,
                    shots_unknown_percentage=shots_unknown_percentage,
                    points_won_percentage=points_won_percentage,
                    points=float(points[count]),
                )
                for count, x in enumerate(player_urls)
            ],
            league=str(league),
            year=dt.year,
            season_type=None,
            end_dt=None,
            attendance=None,
            postponed=None,
            play_off=None,
            distance=None,
            dividends=[],
            pot=None,
            version=version,
            umpires=[],
        )
    except ValueError as exc:
        logging.warning(str(exc))
        logging.warning("url: %s", url)
        raise exc


@MEMORY.cache(ignore=["session"])
def _cached_create_tennisabstract_game_model(
    session: ScrapeSession,
    url: str,
    league: League,
    version: str,
) -> GameModel:
    return _create_tennisabstract_game_model(
        session=session,
        url=url,
        league=league,
        version=version,
    )


def create_tennisabstract_game_model(
    session: ScrapeSession,
    url: str,
    league: League,
) -> GameModel:
    """Create a TennisAbstract game model."""
    if not pytest_is_running.is_running():
        return _cached_create_tennisabstract_game_model(
            session=session,
            url=url,
            league=league,
            version=VERSION,
        )
    with session.cache_disabled():
        return _create_tennisabstract_game_model(
            session=session,
            url=url,
            league=league,
            version=VERSION,
        )



"""HKJC HKJC owner model."""

import logging
import urllib.parse
from urllib.parse import urlparse

from ...owner_model import VERSION, OwnerModel


def create_hkjc_hkjc_owner_model(
    url: str,
) -> OwnerModel | None:
    """Create an HKJC owner."""
    o = urlparse(url)
    query = urllib.parse.parse_qs(o.query)
    try:
        name = query["HorseOwner"][0]
    except KeyError as exc:
        logging.debug(url)
        logging.debug(str(exc))
        return None
    return OwnerModel(
        identifier=name,
        name=name,
        version=VERSION,
    )



"""IPL Sports DB league model."""

from scrapesession.scrapesession import ScrapeSession  # type: ignore

from ...league import League
from ...sportsdb.sportsdb_league_model import SportsDBLeagueModel


class IPLSportsDBLeagueModel(SportsDBLeagueModel):
    """IPL SportsDB implementation of the league model."""

    def __init__(self, session: ScrapeSession, position: int | None = None) -> None:
        super().__init__(session, "4460", League.IPL, position=position)

    @classmethod
    def name(cls) -> str:
        return "ipl-sportsdb-league-model"



"""ESPNCricInfo venue model."""

# pylint: disable=duplicate-code
import datetime
from typing import Any

import pytest_is_running
from scrapesession.scrapesession import ScrapeSession  # type: ignore

from ....cache import MEMORY
from ...google.google_address_model import create_google_address_model
from ...venue_model import VERSION, VenueModel


def _create_espncricinfo_venue_model(
    grounds: dict[str, Any],
    session: ScrapeSession,
    dt: datetime.datetime,
    version: str,
) -> VenueModel:
    venue_name = grounds["name"]
    address_model = create_google_address_model(venue_name, session, dt)
    return VenueModel(
        identifier=str(grounds["id"]),
        name=venue_name,
        address=address_model,
        is_grass=None,
        is_indoor=None,
        is_turf=None,
        is_dirt=None,
        version=version,
    )


@MEMORY.cache(ignore=["session"])
def _cached_create_espncricinfo_venue_model(
    grounds: dict[str, Any],
    session: ScrapeSession,
    dt: datetime.datetime,
    version: str,
) -> VenueModel:
    return _create_espncricinfo_venue_model(
        grounds=grounds, session=session, dt=dt, version=version
    )


def create_espncricinfo_venue_model(
    grounds: dict[str, Any],
    session: ScrapeSession,
    dt: datetime.datetime,
) -> VenueModel:
    """Create a sports reference venue model."""
    if not pytest_is_running.is_running():
        return _cached_create_espncricinfo_venue_model(
            grounds=grounds, session=session, dt=dt, version=VERSION
        )
    with session.cache_disabled():
        return _create_espncricinfo_venue_model(
            grounds=grounds, session=session, dt=dt, version=VERSION
        )



"""The prototype class for a dividend."""

from typing import Literal

from pydantic import BaseModel, ConfigDict, Field

from .field_type import TYPE_KEY, FieldType

DIVIDEND_POOL_COLUMN: Literal["pool"] = "pool"
DIVIDEND_COMBINATION_COLUMN: Literal["combination"] = "combination"
DIVIDEND_DIVIDEND_COLUMN: Literal["dividend"] = "dividend"


class DividendModel(BaseModel):
    """The serialisable dividend class."""

    model_config = ConfigDict(
        validate_assignment=False,
        revalidate_instances="never",
        extra="ignore",
        from_attributes=False,
    )

    pool: str = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.CATEGORICAL},
        alias=DIVIDEND_POOL_COLUMN,
    )
    combination: list[str] = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=DIVIDEND_COMBINATION_COLUMN,
    )
    dividend: float = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=DIVIDEND_DIVIDEND_COLUMN,
    )
